(* Cmd_login *)
(* Copyright (C)2000-2006 Berke Durak                               *)
(* Released under the GNU Lesser General Public License version 2.1 *)

open Unix
open Log
open Html
open Http
open Debug
open Db
open Command
open Sessions
open Globals
module C = Configuration
;;

(*** sanitize *)
let sanitize =
  let rex = Pcre.regexp "^\\s*(\\S+.*\\S+)\\s*$" in
  fun w ->
    try
      let ss = Pcre.exec ~rex w in
      Pcre.get_substring ss 0
    with
    | Not_found -> ""
;;
(* ***)
(*** bundle *)
let bundle : bundle_generator = fun db config ->
  let global = !Globals.global in
  let sessions = global#sessions in
  object(self)
    (*** intializer *)
    initializer ()
    (* ***)
    (*** enter *)
    method private enter ?session form =
      let login = Form.get form "l"
      and password = Form.get form "p"
      in
      let error msg =
        let _,doc = self#login form in
        "Login error", Seq[P(T msg); doc]
      in
      try
        let sid = Users.login ~sessions ~login ~password () in
        (*let sinfo = Users.get_session_info ~sessions ~sid () in*)
        raise (Command.Redirect(Form.add form "sid" sid, "welcome"))
      with
      | Users.Unknown_user -> error (sf "Unknown login %S" login)
      | Users.Bad_password -> error "Bad password."
    (* ***)
    (*** welcome *)
    method private welcome ~session:((user,si) as sinfo) form =
      "Welcome",
      Seq[
        P(T (sf "Dear %s, welcome !" user.Users.u_name));
        P(Seq[T "You may want to do one of the following things:";
              UL[
                Anchor(Link.slink ~sinfo Link.console,   T "Try the EDOS Console,");
                Anchor(Link.slink ~sinfo Link.dql_help,  T "Learn about DQL,");
                Anchor(Link.slink ~sinfo Link.who,       T "See who is already logged in.");
              ]
           ]
        )
        ]
    (* ***)
    (*** logout *)
    method private logout ~session:(user, s_info) form =
      Users.logout ~sessions ~login:user.Users.u_login ();
      raise (Command.Redirect(Form.empty, "index"))
    (* ***)
    (*** who *)
    method private who ?(all=true) ~session:(user, s_info) form =
      "Users logged in",
      Seq[
        let users =
          List.fold_left
            begin fun result (login,name,logged_in) ->
              if logged_in then
                (T(sf "%s (%s)" name login)) :: result
              else
                if all then
                  (T(sf "[Not logged in: %s (%s)]" name login)) :: result
                else
                  result
            end
            []
            (Users.who sessions ())
        in
        match users with
        | [] -> T "No users are logged in."
        | users -> Seq[ T "Users logged in:"; UL users ]
        ]
    (* ***)
    (*** register *)
    method private register ?session form =
      let fresh ?msg ?(login="") ?(name="") ?(p1="") ?(p2="") () =
        "Register",
        Seq[
          begin
            match msg with
            | None ->
              P(Seq[
                T "By registering, you will be able to access to query the EDOS Debian Metadata Database using the Debian \
                   Query Language.  Registration is required for the following reasons but is essentially anonymous (feel free \
                   to enter bogus names - technical connection information (IPs, HTTP headers) are logged anyway):";
                UL[
                  T "To limit the usage of computational resources (as some queries can be expensive to compute),";
                  T "To allow users to have their own variable environment,";
                  T "And to track usage information for improving the query language."]])
            | Some msg -> T msg
          end;
          Form(GET, "register",
            Seq[
              T "Your login (3 characters minimum): ";
              I_text     ("l", login, Some 32, Some 64);
              BR;
              T "Your display name: ";
              I_text     ("n", name,  Some 64, Some 256);
              BR;
              T "Your password (5 characters minimum): ";
              I_password ("p1", p1, Some 8, Some 16);
              BR;
              T "Repeat your password: ";
              I_password ("p2", p2, Some 8, Some 16);
              BR;
              I_button   ("r", "Register");
            ])]
      in
      try
        let _ = Form.get_set form "r" in
        let login = sanitize (Form.get form "l") in
        let name = sanitize (Form.get form ~default:"" "n") in
        let p1 = Form.get form ~default:"" "p1" in
        let p2 = Form.get form ~default:"" "p2" in
        let f msg = fresh ~msg ~login ~name ~p1 ~p2 () in
        if Users.check_login_availability ~sessions ~login () then
          begin
            if p1 = "" then f "Empty password"
            else if p2 = "" then f "Please repeat your password"
            else if p1 <> p2 then f "Passwords don't match"
            else if String.length p1 < 5 then f (sf "Password %S too short" p1)
            else if String.length name < 5 then f "Name too short, must be 5 characters or more"
            else
              begin
                Users.register
                  ~sessions
                  ~login
                  ~name
                  ~password:p1
                  ~privilege_level:0
                  ();
                Users.save ~sessions ~fn:global#users_fn ();
                let sid = Users.login ~sessions ~login ~password:p1 () in
                raise (Command.Redirect(Form.add form "sid" sid, "welcome"))
                (*"Welcome", T (sf "Welcome, your session ID is %S" s_id)*)
              end
          end
        else
          f (sf "Login %S is already taken, try again." login)
      with
      | Not_found -> fresh ()
    (* ***)
    (*** login *)
    method private login ?session form =
      let login = Form.get form ~default:"" "l" in
      let title = "Login" in
      let doc =
        P(Seq[T "You can "; Anchor(Link.link Link.register, T "register as a new user"); T " or log in:";
              Form(GET, "enter",
                Seq[
                  T "Login: ";
                  I_text     ("l", login, Some 32, Some 64);
                  BR;
                  T "Password: ";
                  I_password ("p", "", Some 32, Some 64);
                  BR;
                  I_button   ("e", "Login as an existing user");
                  I_button   ("n", "Register as a new user");
                ])])
      in
      title,doc
    (* ***)
    (*** index *)
    method private index ?session form =
      "Welcome to the EDOS Console.",
      Seq[
        P(T "The EDOS Console allows you to query our servers for information on Debian metadata using a powerful query language, \
             the Debian Query Language (DQL).  Metadata is updated daily since January 1st, 2006.");
        P(Seq[
            T "To use the console, simply ";
            Anchor(Link.link Link.register,
                   T "register");
            T " by giving a login and a user name, and then click on ``Console'' to start using the console right away."])]
    (* ***)
    (*** credits *)
    method private credits ?session form =
      "Credits",
      Seq[
        P(T "The EDOS project is funded by the European Commission as a STREP project under the IST activities of the 6th Framework Programme.");
        P(Seq[T "This application is written in the wondrous OCaml programming language by:";
            UL[
              Seq[T "Berke Durak (INRIA, project Gallium):";
                  UL[T "Web interface";
                     T "Database";
                     T "Query language design";
                     T "Query engine";
                     T "And all the rest"]];
              Seq[T "Jérôme Vouillon (PPS, Université Paris 7):";
                  UL[T "Problem-specific SAT solver"]]]])]
    (* ***)
    method shutdown = ()
    method table = [
      "login",      Html(Anonymous self#login);
      "logout",     Html(Logged_in self#logout);
      "welcome",    Html(Logged_in self#welcome);
      "index",      Html(Anonymous self#index);
      "register",   Html(Anonymous self#register);
      "who",        Html(Logged_in (self#who ~all:true));
      "enter",      Html(Anonymous self#enter);
      "credits",    Html(Anonymous self#credits);
    ]
end
;;
(* ***)
